<?php

namespace Teknisa\TCO\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class Tco01100PropostaComercialService extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $tcoUtil;
    protected $tcoList;
    protected $connection;
    protected $environment;
    protected $crudService;
    protected $entityManager;
    protected $dataSourceName;
    protected $dataSourceManager;
    protected $tco01100GeneralLibPropCom;
    protected $tco01100DocumentoPropostaService;

    public function __construct($crudService, ParameterBag $parameterBag, $environment, $entityManager, $dataSourceManager, $newCode, $tcoUtil, $tcoList, $tco01100GeneralLibPropCom, $tco01100DocumentoPropostaService) {
        $this->tcoUtil                          = $tcoUtil;
        $this->newCode                          = $newCode;
        $this->tcoList                          = $tcoList;
        $this->connection                       = $entityManager->getConnection();
        $this->crudService                      = $crudService;
        $this->environment                      = $environment;
        $this->entityManager                    = $entityManager;
        $this->dataSourceManager                = $dataSourceManager;
        $this->tco01100GeneralLibPropCom        = $tco01100GeneralLibPropCom;
        $this->tco01100DocumentoPropostaService = $tco01100DocumentoPropostaService;
    }

    /**
     * Salva o registro de proposta comercial.
     * Cria uma nova requisição com os dados preenchidos
     * (Data de vigência; Gerente de Negocio; Código e 
     * Numero da proposta comercial) da requisição 
     * atual para que o 'crud' trate corretamente 
     * as colunas não preenchidas.
     * 
     * @param {DTO\Request}  $request  Requisição vinda 
     *  do cliente.
     * @param {DTO\Response} $response Resposta à requisição
     *  vinda do cliente.
     */
    public function saveProposta(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow()->getArrayCopy();
        unset($row['$error']);

        $nrOrg                = $this->environment->getNrOrg();
        $cdOperador           = $this->environment->getCdOperador();
        $nrGerenteNegocio     = $this->getGerenteNegocio($cdOperador)['NRGERENTENEGOCIO'];
        $nrPropostaComercial  = $this->newCode->getCodeByTableName('PROPOSTACOMERCIAL', $nrOrg, 1, 10, true);
        $cdPropostaComercial  = str_pad($nrPropostaComercial, 10, "0", STR_PAD_LEFT);
        $nrResponsavelPropCom = $this->newCode->getCodeByTableName('RESPONSAVELPROPCOM', $nrOrg, 1, 10, true);

        $row['DTINIVIGENCIA']        = $row['VIGENCIA']['start'];
        $row['DTFIMVIGENCIA']        = $row['VIGENCIA']['end'];
        $row['NRGERENTENEGOCIO']     = $nrGerenteNegocio;
        $row['CDPROPOSTACOMERCIAL']  = $cdPropostaComercial;
        $row['NRPROPOSTACOMERCIAL']  = $nrPropostaComercial;
        $row['NRRESPONSAVELPROPCOM'] = $nrResponsavelPropCom;

        $reqRPath   = $request->getRoutePath();
        $reqMethod  = $request->getMethod();
        $reqUserId  = $request->getUserId();

        $reqDtSet  = new DataSet('propostacomercial', array($row));
        $request   = new DTO\Request\DataSet($reqDtSet, $reqMethod, $reqRPath, $reqUserId);
        $rowToSave = $this->crudService->getRowsToSave($request, 'propostacomercial');
        $dataSet   = new DataSet('propostacomercial', $rowToSave);
        $this->dataSourceManager->persist($dataSet);
        
        $reqDtSet  = new DataSet('responsavelpropcom', array($row));
        $request   = new DTO\Request\DataSet($reqDtSet, $reqMethod, $reqRPath, $reqUserId);
        $rowToSave = $this->crudService->getRowsToSave($request, 'responsavelpropcom');
        $dataSet   = new DataSet('responsavelpropcom', $rowToSave);
        $this->dataSourceManager->persist($dataSet);
        
        $columnsToReturn = array(
            "NRPROPOSTACOMERCIAL"=>$nrPropostaComercial,
            "CDPROPOSTACOMERCIAL"=>$cdPropostaComercial
        );
        $response->addDataSet(new DataSet('columnsToReturn', $columnsToReturn));
    }

    /**
     * Retorna o gerente de negocio de acordo com o código
     * do operador passado por parâmetro.
     * 
     * @param {String} $cdOperador Código do operador.
     * 
     * @return {Array} Gerente de negócio correspondente.
     */
    public function getGerenteNegocio($cdOperador = ''){
        return $this->tco01100GeneralLibPropCom->getGerenteNegocio($cdOperador);
    }

    /**
     * Retorna o estado de existência e validade
     * do endereço passado por parâmetro.
     * 
     * @param {String} $cdPais      Código do Pais.
     * @param {String} $sgEstado    Código do Estado.
     * @param {String} $cdMunicipio Código do Municipio.
     * 
     * @return {Array} Gerente de negócio correspondente.
     */
    public function isValidEndereco($cdPais, $sgEstado, $cdMunicipio){
        return $this->tco01100GeneralLibPropCom->isValidEndereco($cdPais, $sgEstado, $cdMunicipio);
    }

    /**
     * Verifica se os parametros de cliente/prospect;
     * segmento; ramo de atividade e tipo de contrato
     * da proposta comercial possuem idativo = 'S'.
     * 
     * @param array $register Registro com detalhes
     *  da proposta comercial.
     * 
     * @return bool Retorna true se todos os dados
     *  possuem idativo = 'S'.
     */
    public function isActiveParamsPropCom($register){
        $cliente = $register->has('CDCLIENTE') ? $register->get('CDCLIENTE') ?: false : false;
        $prospect = $register->has('CDPROSPECT') ? $register->get('CDPROSPECT') ?: false : false;
        $segmento = $register->has('CDSEGMER') ? $register->get('CDSEGMER') ?: false : false;
        $ramoAtiv = $register->has('CDATIVCLI') ? $register->get('CDATIVCLI') ?: false : false;
        $tipoCont = $register->has('NRTIPOCONTRATO') ? $register->get('NRTIPOCONTRATO') ?: false : false;
        
        if(!is_bool($cliente)) $cliente = $this->tco01100GeneralLibPropCom->getData('cliente', array('CDCLIENTE'=>$cliente))[0];
        if(!is_bool($prospect)) $prospect = $this->tco01100GeneralLibPropCom->getData('prospect', array('CDPROSPECT'=>$prospect))[0];
        $segmento = $this->tco01100GeneralLibPropCom->getData('segmercado', array('CDSEGMER'=>$segmento))[0];
        $ramoAtiv = $this->tco01100GeneralLibPropCom->getData('ativcliente', array('CDATIVCLI'=>$ramoAtiv))[0];
        $tipoCont = $this->tco01100GeneralLibPropCom->getData('tipocontrato', array('NRTIPOCONTRATO'=>$tipoCont))[0];
        
        if(!is_bool($cliente)) {
            $cliente = $cliente['IDATIVO']=='S';
        } else {
            $cliente = true;
        }

        if(!is_bool($prospect)) {
            $prospect = $prospect['IDATIVO']=='S';
        } else {
            $prospect = true;
        }

        $segmento = $segmento['IDATIVO']=='S';
        $ramoAtiv = $ramoAtiv['IDATIVO']=='S';
        $tipoCont = $tipoCont['IDATIVO']=='S';

        return $cliente && $prospect &&
            $segmento && $ramoAtiv && $tipoCont;
    }

    /**
     * Função responsável por retornar o código de versão da
     * proposta comercial livre a ser utilizado na versão.
     * 
     * @param {String} $nrPropostaComercial Número da proposta
     *  comercial a ser usada.
     * 
     * @return {String} Código gerado.
     */
    public function getCdVersaoPropCom($nrPropostaComercial){
        $sql = "SELECT
                    NVL(LPAD(MAX(CDVERSAOPROPCOM) + 1,3,'0'), '001') GCODE
                FROM
                    VERSAOPROPCOM 
                WHERE
                    NRPROPOSTACOMERCIAL = :NRPROPOSTACOMERCIAL";

        $params = array(
            "NRPROPOSTACOMERCIAL" => $nrPropostaComercial
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll()[0]['GCODE'];

        return $result;
    }

    /**
     * Verifica se os parametros de serviço, padrão de
     * qualidade e tipo de publico do serviço da
     *  proposta comercial possuem idativo = 'S'.
     * 
     * @param array $register Registro com detalhes
     *  do serviço da proposta comercial.
     * 
     * @return bool Retorna true se todos os dados
     *  possuem idativo = 'S'.
     */
    public function isActiveParamsServicoProp($register){
        $servico = $register->has('NRSERVICOPROPOSTA') ? $register->get('NRSERVICOPROPOSTA') ?: false : false;
        $padrQual = $register->has('NRPADRAOQUALIDADE') ? $register->get('NRPADRAOQUALIDADE') ?: false : false;
        $publico = $register->has('NRTIPOPUBLICO') ? $register->get('NRTIPOPUBLICO') ?: false : false;
        
        $servico = $this->tco01100GeneralLibPropCom->getData('tco01100_servico_proposta', array('NRSERVICOPROPOSTA'=>$servico))[0];
        $padrQual = $this->tco01100GeneralLibPropCom->getData('tco01100_padrao_qualidade', array('NRPADRAOQUALIDADE'=>$padrQual))[0];
        $publico = $this->tco01100GeneralLibPropCom->getData('tipopublico', array('NRTIPOPUBLICO'=>$publico))[0];
        
        $servico = $servico['IDATIVO']=='S';
        $padrQual = $padrQual['IDATIVO']=='S';
        $publico = $publico['IDATIVO']=='S';

        return $servico && $padrQual && $publico;
    }

    /**
     * Adiciona nova versão à proposta comercial passada
     * por parâmetro.
     * 
     * @param {DTO\Request}  $request  Requisição vinda 
     *  do cliente.
     * @param {DTO\Response} $response Resposta à requisição
     *  vinda do cliente.
     */
    public function newVersion(DTO\Request $request, DTO\Response $response){
        $row                 = $request->getRow();
        $nrOrg               = $this->environment->getNrOrg();
        $nrPropostaComercial = $row->get('NRPROPOSTACOMERCIAL');
        $cdVersaoPropCom     = $this->getCdVersaoPropCom($nrPropostaComercial);
        $nrVersaoPropCom     = $this->newCode->getCodeByTableName('VERSAOPROPCOM', $nrOrg, 1, 10, true);

        $row->set('IDVERSAOPROPCOM', 'PENDENTE');
        $row->set('CDVERSAOPROPCOM', $cdVersaoPropCom);
        $row->set('NRVERSAOPROPCOM', $nrVersaoPropCom);
        $row->set('__is_new', true);
        
        $reqRPath   = $request->getRoutePath();
        $reqMethod  = $request->getMethod();
        $reqUserId  = $request->getUserId();

        $request   = new DTO\Request\Row($row, $reqMethod, $reqRPath, $reqUserId);
        $rowToSave = $this->crudService->getRowsToSave($request, 'versaopropcom');
        $dataSet   = new DataSet('versaopropcom', $rowToSave);
        $this->dataSourceManager->persist($dataSet);
    }

    /**
     * Valida se o operador passado por parâmetro é um 
     * responsável válido, ou respectivo lider, pela 
     * proposta comercial.
     * 
     * @param {String} $cdOperador Código do Operador.
     * @param {String} $nrPropostaComercial Número da proposta
     *  comercial.
     */
    public function isValidResponsavelPropCom($cdOperador, $nrPropostaComercial){
        return $this->tco01100GeneralLibPropCom->isValidResponsavelPropCom($cdOperador, $nrPropostaComercial);
    }
    
    /**
     * Retorna proposta comercial de acordo com os parametros
     * 
     *
     * @param {Array} $params Parametros para consulta
     *  de proposta comercial.
     */
    public function findPropostaComercial($params, $conditions = null){
         $valuesWithDistinctLabel = array(
            "OPERADORPLANEJAMENTO" => "OPENMOPERADOR"    
        );
        $assocFieldToTable = array(
            "CDPROPOSTACOMERCIAL" => array(
                'tableName' => 'PC'
            ),
            "NMGERENTENEGOCIO" => array(
                'tableName' => 'O',
                "format" => "( tableName.CDOPERADOR || ' - ' || tableName.NMOPERADOR )"
            ),
            "CLIENTEPROSPECT" => array(
                'tableName' => "",
                'format' => "(NVL(C.NRINSJURCLIE,P.NRINSJURPROS) || ' - ' || NVL(C.NMRAZSOCCLIE,P.NMRAZSOCPROS))"
            ),
            "DTLANCAMENTO" => array(
                "tableName" => "PC",
                "format" => "TO_CHAR(tableName.NAME_COLUMN,'DD/MM/YYYY HH24:MI:SS')"
            ),
            "NRVERSAOPROPCOM" => array(
                "tableName" => "",
                "format" => "(SELECT MAX(CDVERSAOPROPCOM) FROM VERSAOPROPCOM VE WHERE VE.NRPROPOSTACOMERCIAL = PC.NRPROPOSTACOMERCIAL AND VE.IDVERSAOPROPCOM = 'APROVADA')"
            ),
            "SITUACAO" => array(
                "tableName" => "PC",
                "format" => "SIT_PROPCOMERCIAL(tableName.NRPROPOSTACOMERCIAL)"
            ),
            "DTULTATU" => array(
                "tableName" => "PC",
                "format" => "TO_CHAR(tableName.NAME_COLUMN,'DD/MM/YYYY HH24:MI:SS')"
            ),
            "NMRESPONSAVEL" => array(
                "tableName" => "OPERRESP",
                "format"    => "tableName.CDOPERADOR || ' - ' || tableName.NMOPERADOR"
            )
            
        );
        $queryLikeAll = $this->tcoUtil->prepareQueryLikeAll($conditions, $assocFieldToTable, $valuesWithDistinctLabel);
        $sql = "SELECT PC.*,
                       (NVL(C.NRINSJURCLIE,P.NRINSJURPROS) || ' - ' || NVL(C.NMRAZSOCCLIE,P.NMRAZSOCPROS)) CLIENTEPROSPECT,
                       NVL(C.NRINSJURCLIE,P.NRINSJURPROS) CDCLIENTEPROSPECT,
                       ( O.CDOPERADOR || ' - ' || O.NMOPERADOR ) NMGERENTENEGOCIO,
                       PA.NMPAIS,
                       ES.NMESTADO,
                       MN.NMMUNICIPIO,
                       RE.NMREGIAO,
                       SGM.DSSEGMER SEGMENTO,
                       ATC.DSATIVCLI RAMOATIVIDADE,
                       TC.NMTIPOCONTRATO TIPOCONTRATO,
                       O.CDOPERADOR,
                       OPERRESP.CDOPERADOR || ' - ' || OPERRESP.NMOPERADOR NMRESPONSAVEL,
                       (SELECT MAX(CDVERSAOPROPCOM) 
                          FROM VERSAOPROPCOM VE 
                         WHERE VE.NRPROPOSTACOMERCIAL = PC.NRPROPOSTACOMERCIAL 
                           AND VE.IDVERSAOPROPCOM = 'APROVADA') AS NRVERSAOPROPCOM,
                       SIT_PROPCOMERCIAL(PC.NRPROPOSTACOMERCIAL) AS SITUACAO
                  FROM PROPOSTACOMERCIAL PC
                 INNER JOIN GERENTENEGOCIO G ON G.NRGERENTENEGOCIO = PC.NRGERENTENEGOCIO
                 INNER JOIN OPERADOR O ON O.CDOPERADOR = G.CDOPERADOR
                  LEFT JOIN CLIENTE C ON C.CDCLIENTE = PC.CDCLIENTE
                  LEFT JOIN PROSPECT P ON P.CDPROSPECT = PC.CDPROSPECT
                  LEFT JOIN PAIS PA ON PA.CDPAIS = PC.CDPAIS
                  LEFT JOIN ESTADO ES 
                    ON ( ES.SGESTADO = PC.SGESTADO
                   AND ES.CDPAIS = PA.CDPAIS )
                  LEFT JOIN MUNICIPIO MN 
                    ON ( MN.SGESTADO = ES.SGESTADO
                   AND MN.CDPAIS = PA.CDPAIS
                   AND MN.CDMUNICIPIO = PC.CDMUNICIPIO )
                  LEFT JOIN REGIAO RE 
                    ON ( MN.CDREGIAO = RE.CDREGIAO )
                  LEFT JOIN SEGMERCADO SGM 
                    ON SGM.CDSEGMER = PC.CDSEGMER
                  LEFT JOIN ATIVCLIENTE ATC 
                    ON ATC.CDATIVCLI = PC.CDATIVCLI
                  LEFT JOIN TIPOCONTRATO TC 
                    ON TC.NRTIPOCONTRATO = PC.NRTIPOCONTRATO
                  LEFT JOIN RESPONSAVELPROPCOM RSPC 
                    ON RSPC.NRPROPOSTACOMERCIAL = PC.NRPROPOSTACOMERCIAL
                  LEFT JOIN GERENTENEGOCIO GERRESP 
                    ON GERRESP.NRGERENTENEGOCIO = RSPC.NRGERENTENEGOCIO
                  LEFT JOIN OPERADOR OPERRESP 
                    ON OPERRESP.CDOPERADOR = GERRESP.CDOPERADOR
                 WHERE CDPROPOSTACOMERCIAL LIKE '%' ||:CDPROPOSTACOMERCIAL || '%'
                   AND ( (PC.CDCLIENTE IS NOT NULL AND VALOR_NA_LISTA(PC.CDCLIENTE, :CLIENTES) > 0) 
                          OR (PC.CDPROSPECT IS NOT NULL AND VALOR_NA_LISTA(PC.CDPROSPECT, :PROSPECTS) > 0) )
                   AND TRUNC(DTLANCAMENTO) BETWEEN :DTINICIAL AND :DTFINAL
                   AND EXISTS ( SELECT 1
                                  FROM RESPONSAVELPROPCOM R
                                 INNER JOIN TABLE ( LIDERADOGN(:CDOPERADOR) ) L 
                                    ON L.NRGERENTENEGOCIO = R.NRGERENTENEGOCIO
                                 WHERE NRPROPOSTACOMERCIAL = PC.NRPROPOSTACOMERCIAL
                                   AND VALOR_NA_LISTA(TO_CHAR(NRGERENTENEGOCIO),:GERENTES) > 0 )
                   AND (:SITUACAO = 'T' OR SIT_PROPCOMERCIAL(PC.NRPROPOSTACOMERCIAL) = :SITUACAO)
                   $queryLikeAll
                 ORDER BY PC.CDPROPOSTACOMERCIAL";
        $this->setListOnParam($params['GERENTES']);
        $this->setListOnParam($params['CLIENTES']);
        $this->setListOnParam($params['PROSPECTS']);//var_dump($sql);die;
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        
        if($params['GERENTES'] != 'T') {
            $this->tcoList->apagarListaValores($params['GERENTES']);
        }
        if($params['CLIENTES'] != 'T') {
            $this->tcoList->apagarListaValores($params['CLIENTES']);
        }
        if($params['PROSPECTS'] != 'T') {
            $this->tcoList->apagarListaValores($params['PROSPECTS']);
        }
        
        return $result;
    }

    /**
     * Define a lista no parametro referente usando
     * a classe de lista do TcoUtil.
     * 
     * @param {String|Array} $param Array com valores de
     *  parametros a serem definidos como lista.
     */
    private function setListOnParam(&$param){
        $this->tco01100GeneralLibPropCom->setListOnParam($param);
    }

    /**
     * Salva inserção de serviço de proposta comercial.
     * 
     * @param {DTO\Request}  $request  Requisição vinda 
     *  do cliente.
     * @param {DTO\Response} $response Resposta à requisição
     *  vinda do cliente.
     */
    public function insertServicoPropCom(DTO\Request $request, DTO\Response $response){
        $row                 = $request->getRow();
        $nrOrg               = $this->environment->getNrOrg();
        $nrServicoPropCom     = $this->newCode->getCodeByTableName('SERVICOPROPCOM', $nrOrg, 1, 10, true);

        $row->set('NRSERVICOPROPCOM', $nrServicoPropCom);
        $row->set('IDSERVICOPROPCOM', 'PENDENTE');
        $row->set('__is_new', true);
        
        if(!$row->has('QTTURNOS') || !$row->get('QTTURNOS'))
        $row->set('QTTURNOS', 1);
        
        $reqRPath   = $request->getRoutePath();
        $reqMethod  = $request->getMethod();
        $reqUserId  = $request->getUserId();
        
        $requestRow   = new DTO\Request\Row($row, $reqMethod, $reqRPath, $reqUserId);
        $rowToSave = $this->crudService->getRowsToSave($requestRow, 'servicopropcom');
        $dataSet   = new DataSet('servicopropcom', $rowToSave);
        $this->dataSourceManager->persist($dataSet);
        
        $nrDetalheServPropCom = $this->insertDetalhesServico($requestRow);
        
        $row->set('NRDETALHESERVPROPCOM', $nrDetalheServPropCom);
        $row->set('__is_new', false);
        $requestRow   = new DTO\Request\Row($row, $reqMethod, $reqRPath, $reqUserId);
        $rowToSave = $this->crudService->getRowsToSave($requestRow, 'servicopropcom');
        $dataSet   = new DataSet('servicopropcom', $rowToSave);
        $this->dataSourceManager->persist($dataSet);
        
        $nrCardapioBase = $this->getCardapioBaseByServParams($row);
        $invalidCardBaseServPropCom = $this->setCardapioBaseOnServico($row, $nrCardapioBase);

        $columnsToReturn = array("NRSERVICOPROPCOM"=>$nrServicoPropCom);
        $columnsToReturn["NRDETALHESERVPROPCOM"] = $nrDetalheServPropCom;
        if($invalidCardBaseServPropCom){
            $columnsToReturn['ERROR_CODE'] = $invalidCardBaseServPropCom;
        } else {
            $columnsToReturn['NRCARDAPIOBASE'] = $nrCardapioBase;
        }

        $response->addDataSet(new DataSet('columnsToReturn', $columnsToReturn));
        return [
            'NRSERVICOPROPCOM' => $nrServicoPropCom,
            'NRDETALHESERVPROPCOM' => $nrDetalheServPropCom,
            'NRCARDAPIOBASE' => is_array($nrCardapioBase) ? $nrCardapioBase[0]['NRCARDAPIOBASE'] : ''
        ];
    }
    
    /**
     * Insere detalhe de serviço proposta comercial.
     * 
     * @param DTO\Request\Row $request
     * 
     * @return string $nrDetalheServPropCom
     */
    public function insertDetalhesServico($request){
        $row                 = $request->getRow();
        $nrOrg               = $this->environment->getNrOrg();
        $nrDetalheServPropCom    = $this->newCode->getCodeByTableName('DETALHESERVPROPCOM', $nrOrg, 1, 10, true);
        
        $row->set('NRDETALHESERVPROPCOM', $nrDetalheServPropCom);
        $row->set('__is_new', true);
        
        $reqRPath   = $request->getRoutePath();
        $reqMethod  = $request->getMethod();
        $reqUserId  = $request->getUserId();
        
        $requestRow   = new DTO\Request\Row($row, $reqMethod, $reqRPath, $reqUserId);
        $rowToSave = $this->crudService->getRowsToSave($requestRow, 'detalheservpropcom');
        $dataSet   = new DataSet('detalheservpropcom', $rowToSave);
        $this->dataSourceManager->persist($dataSet);

        return $nrDetalheServPropCom;
    }

    /**
     * Define cardapio base para serviço caso 
     * exista e seja compatível.
     * 
     * @param DTO\Row $row
     * 
     * @return string Mensagem de erro. Se não
     * houver erro ao definir cardapio, retorna
     * false.
     */
    public function setCardapioBaseOnServico($row, $nrCardapioBase){
        $invalidCard = false;

        if($this->tcoUtil->oldsizeof($nrCardapioBase)>1){
            $invalidCard = 'TCO01100_MT_ONE_CARDBASE_NOTIF';
        } else if(!$nrCardapioBase){
            $invalidCard = 'TCO01100_ZERO_CARDBASE_NOTIF';
        } else {
            $sql = "UPDATE SERVICOPROPCOM
                SET NRCARDAPIOBASE = :NRCARDAPIOBASE
                WHERE NRSERVICOPROPCOM = :NRSERVICOPROPCOM";

            $params = array(
                "NRSERVICOPROPCOM"=>$row->get('NRSERVICOPROPCOM'),
                "NRCARDAPIOBASE"=>$nrCardapioBase[0]['NRCARDAPIOBASE']
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
        }

        return $invalidCard;
    }

    /**
     * Retorna cardápio de acordo com parametros
     * do serviço da proposta.
     * 
     * @param DTO\Row $row
     * 
     * @return array|bool Retorna o registro encontrado
     * ou false caso não encontre o registro.
     */
    public function getCardapioBaseByServParams($row){
        $sql = "SELECT C.NRCARDAPIOBASE
                FROM CARDAPIOBASE C
                WHERE
                    C.NRSERVICOPROPOSTA =:NRSERVICOPROPOSTA
                    AND C.NRPADRAOQUALIDADE =:NRPADRAOQUALIDADE
                    AND C.IDATIVO = 'S'
                    AND TRUNC(SYSDATE) BETWEEN C.DTINIVIGENCIA AND C.DTFIMVIGENCIA
                    AND IDCOMPLEMENTAR = 'N'
                AND EXISTS ( SELECT 1
                            FROM SEGMERCARDBASE
                            WHERE
                                NRCARDAPIOBASE = C.NRCARDAPIOBASE
                                AND CDSEGMER = (
                                    SELECT CDSEGMER
                                    FROM PROPOSTACOMERCIAL PC
                                    WHERE NRPROPOSTACOMERCIAL = (
                                            SELECT NRPROPOSTACOMERCIAL
                                            FROM VERSAOPROPCOM
                                            WHERE NRVERSAOPROPCOM =:NRVERSAOPROPCOM
                )))";

        $params = array(
            "NRSERVICOPROPOSTA"=>$row->get('NRSERVICOPROPOSTA'),
            "NRPADRAOQUALIDADE"=>$row->get('NRPADRAOQUALIDADE'),
            "NRVERSAOPROPCOM"=>$row->get('NRVERSAOPROPCOM')
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();

        return $this->tcoUtil->oldsizeof($result) ? $result : false;
    }

    /**
     * Verifica se o Serviço já existe em uma proposta
     * para o padrão de qualidade e tipo de público 
     * correspondente.
     * 
     * @param {Array[String]} $params Array de parametros
     *  para execução da query.
     * 
     * @return {Array|Boolean} Retorna o registro caso
     *  exista e false caso não exista.
     */
    public function getServicoProposta($params){
        $conditions = array(
            array(
                "columnName" => "NRSERVICOPROPOSTA", 
                "operator"   => "=",
                "value"      => $params["NRSERVICOPROPOSTA"]
            ),
            array(
                "columnName" => "NRPADRAOQUALIDADE", 
                "operator"   => "=",
                "value"      => $params["NRPADRAOQUALIDADE"]
            ),
            array(
                "columnName" => "NRTIPOPUBLICO", 
                "operator"   => "=",
                "value"      => $params["NRTIPOPUBLICO"]
            ),
            array(
                "columnName" => "NRVERSAOPROPCOM", 
                "operator"   => "=",
                "value"      => $params["NRVERSAOPROPCOM"]
            )
        );

        $filterCriteria = new FilterCriteria("servicopropcom", $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria)->getRows();

        return $this->tcoUtil->oldsizeof($dataSet) ? $dataSet[0] : false;
    }

    /**
     * Remove registro de serviço de proposta
     * comercial.
     * 
     * @param {Array} $rows Array de códigos dos serviços da
     *  proposta a serem removidos.
     */
    public function removeServicoProposta($rows){
        foreach($rows as $row){
            $sql = "BEGIN
            DELETA_SERVICOPROPCOM(:SERVICOPROPCOM);
            END;";
            
            $params = array(
                "SERVICOPROPCOM" => $row
            );
            
            try{
                $query = $this->entityManager->getConnection()->prepare($sql);
                $query->execute($params);
            } catch(Exception $e){
                throw $e;
            }
        }
    }

    
    /**
     * Remove registro de serviço de proposta
     * comercial.
     * 
     * @param {Array} $rows Array de códigos dos serviços da
     *  proposta a serem removidos.
     */
    public function removeVersion($rows){
        try{
            $sql = "DELETE FROM VERSAOPROPCOM
                WHERE 
                VALOR_NA_LISTA(TO_CHAR(NRVERSAOPROPCOM),:VERSAOPROPCOM) > 0";

            $this->setListOnParam($rows);
            
            $params = array(
                "VERSAOPROPCOM" => $rows
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            
            $this->tcoList->apagarListaValores($rows);
        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * Remove registro de proposta comercial.
     * 
     * @param {String} $nrPropostaComercial Código da proposta 
     *  a ser removidos.
     */
    public function removeProposta($nrPropostaComercial){
        try{
            $this->removeAllDocPropCom($nrPropostaComercial);
            $sql = "BEGIN
                DELETE FROM RESPONSAVELPROPCOM  WHERE NRPROPOSTACOMERCIAL = :NRPROPOSTACOMERCIAL;
                DELETE FROM PROPOSTACOMERCIAL   WHERE NRPROPOSTACOMERCIAL = :NRPROPOSTACOMERCIAL;
            END;";

            $params = array(
                "NRPROPOSTACOMERCIAL" => $nrPropostaComercial
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * Exclui todos os documentos da proposta comercial.
     * 
     * @param {Integer} $nrPropostaComercial Número da 
     *  proposta comercial.
     */
    private function removeAllDocPropCom($nrPropostaComercial){
        $docsPropCom = $this->getAllDocPropCom($nrPropostaComercial);
        
        if($docsPropCom){
            $docsPropCom = array_map(function($reg){
                return $reg['NRDOCUMENTOPROPCOM'];
            }, $docsPropCom);

            $this->tco01100DocumentoPropostaService->removeFile($docsPropCom);
        }
    }

    /**
     * Retorna todos os documentos da proposta comercial
     * passada por parâmetro.
     * 
     * @param {String} $nrPropostaComercial Numero da 
     *  proposta comercial.
     */
    private function getAllDocPropCom($nrPropostaComercial){
        $conditions = array(
            array(
                "columnName" => "NRPROPOSTACOMERCIAL", 
                "operator"   => "=",
                "value"      => $nrPropostaComercial
            )
        );

        $filterCriteria = new FilterCriteria("documentopropcom_reduced", $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria)->getRows();

        return $this->tcoUtil->oldsizeof($dataSet) ? $dataSet : false;
    }

    /**
     * Retorna as dependencias das versões de 
     * proposta comercial passadas por parâmetros.
     * 
     * @param {Array} $versions Códigos de versões
     *  de propostas comerciais.
     */
    public function getVersionDependencies($versions){
        try{
            $sql = "SELECT
                        'TCO01100_VERSAO_POSSUI_SERV' MESSAGECODE,
                        SPC.NRVERSAOPROPCOM,
                        SP.CDSERVICOPROPOSTA PARAM1,
                        SP.NMSERVICOPROPOSTA PARAM2,
                        VPC.CDVERSAOPROPCOM PARAM3
                    FROM
                        SERVICOPROPCOM SPC
                        LEFT JOIN SERVICOPROPOSTA SP ON SPC.NRSERVICOPROPOSTA = SP.NRSERVICOPROPOSTA
                        LEFT JOIN VERSAOPROPCOM VPC ON VPC.NRVERSAOPROPCOM = SPC.NRVERSAOPROPCOM
                    WHERE
                        VALOR_NA_LISTA(TO_CHAR(SPC.NRVERSAOPROPCOM),:NRVERSAOPROPCOM)>0";
            
            $this->setListOnParam($versions);
            
            $params = array(
                "NRVERSAOPROPCOM" => $versions
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            $result = $query->fetchAll();

            $this->tcoList->apagarListaValores($versions);
            return $result;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * Retorna as dependencias da proposta comercial
     * passadas por parâmetros.
     * 
     * @param {Int} $nrPropostaComercial Número da
     *  proposta comercial.
     */
    public function getPropostaDependencies($nrPropostaComercial){
        try{
            $sql = "SELECT 
                        'TCO01100_PROPOSTA_POSSUI_VERS' MESSAGECODE,
                        PC.CDPROPOSTACOMERCIAL PARAM1,
                        VPC.CDVERSAOPROPCOM PARAM2
                    FROM VERSAOPROPCOM VPC
                        LEFT JOIN PROPOSTACOMERCIAL PC ON VPC.NRPROPOSTACOMERCIAL = PC.NRPROPOSTACOMERCIAL
                    WHERE 
                        PC.NRPROPOSTACOMERCIAL = :NRPROPOSTACOMERCIAL";
            
            $params = array(
                "NRPROPOSTACOMERCIAL" => $nrPropostaComercial
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);

            return $query->fetchAll();

        } catch(Exception $e){
            throw $e;
        }
    }


    /**
     * Retorna versao da proposta.
     * 
     * @param int|float $nrVersao
     */
    public function getVersao($nrVersao){
        return $this->tco01100GeneralLibPropCom->getData('versaopropcom', array(
            'NRVERSAOPROPCOM'=>$nrVersao
        ));
    }
    
    /**
     * Retorna numero do cardapio salvo no servico da
     * proposta.
     * 
     * @param DTO\Row $row
     */
    public function getServPropComCard($row){
        try{
            $sql = "SELECT NRCARDAPIOBASE
                FROM SERVICOPROPCOM
                WHERE
                    NRSERVICOPROPCOM = :NRSERVICOPROPCOM 
                    AND NRCARDAPIOBASE IS NOT NULL";
            
            $params = array(
                "NRSERVICOPROPCOM" => $row->get('NRSERVICOPROPCOM')
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            $result = $query->fetchAll();
            
            return $this->tcoUtil->oldsizeof($result) ? $result[0] : false;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * Altera registro de responsavel pela
     * proposta comercial.
     * 
     * @param DTO\Row $row
     * @param array $reqParams Parametros da
     * requisição.
     */
    public function changeResponsavelPropCom($row, $reqParams){
        $responsavelPropCom = $this->tco01100GeneralLibPropCom->getData('responsavelpropcom', array(
            'NRPROPOSTACOMERCIAL'=>$row->get('NRPROPOSTACOMERCIAL')
        ));
        
        if($responsavelPropCom){
            $row->set('__is_new', false);
            $row->set('NRRESPONSAVELPROPCOM', $responsavelPropCom[0]['NRRESPONSAVELPROPCOM']);
            
            $dataSet   = new DataSet('responsavelpropcom', array($row));
            $this->dataSourceManager->delete($dataSet);
        }

        $nrOrg                = $this->environment->getNrOrg();
        $nrResponsavelPropCom  = $this->newCode->getCodeByTableName('RESPONSAVELPROPCOM', $nrOrg, 1, 10, true);
    
        $row->set('__is_new', true);
        $row->set('NRRESPONSAVELPROPCOM', $nrResponsavelPropCom);
        
        $request   = new DTO\Request\Row($row, $reqParams['REQMETHOD'], $reqParams['REQRPATH'], $reqParams['REQUSERID']);
        $rowToSave = $this->crudService->getRowsToSave($request, 'responsavelpropcom');
        $dataSet   = new DataSet('responsavelpropcom', $rowToSave);
        $this->dataSourceManager->persist($dataSet);
    }


    /**
     * Retorna dados completos do detalhe do serviço.
     * 
     * @param object $row parametros para consulta.
     */
    public function getDetalheData($row){
        return $this->tco01100GeneralLibPropCom->getServPropComComplete($row->get('NRSERVICOPROPCOM'));
    }

    /**
     * Rotina de notificar planejamento, salvando
     * dados da tabela "HISTNOTIFPROPCOM".
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function notificarPlan(DTO\Request $request, DTO\Response $response, $row){
        $row = $request->getRow();
        $row->set('IDORIGEM', 'GN');
        $row->set('IDDESTINO', 'PLANEJAMENTO');
        $row->set('IDMOTIVO', $row->get('MOTIVO'));

        $procRet = $this->tco01100GeneralLibPropCom->notificarPlan($row, $request);

        if($procRet instanceof Error){

            $procRet = explode(' ', $procRet->getMessage());
            $errorMsg = array_values(array_filter($procRet, function($word){
                return strpos($word, '_MSGSTA_')!==false;
            }))[0];

            $errorMsg = explode("\n", $errorMsg);
            $errorMsg = array_filter($errorMsg, function($word){
                return strpos($word, '_MSGSTA_')!==false;
            })[0];

            $errorMsg = str_replace('_MSGSTA_', '', $errorMsg);
            $errorMsg = str_replace('_MSGEND_', '', $errorMsg);

            $response->setError(new Error($errorMsg, -1));
        }
    }

    /**
     * Conclui versão selecionada na tela de
     * versão.
     * 
     * @param object $row
     * 
     * @return string|array 'VALID' caso a operação
     * seja concluida com sucesso, array de erros
     * caso não seja.
     */
    public function concludeVersion($row){
        $validVerToConclude = $this->isValidVersaoToConclude($row, 'CONCLUIDA');
        
        if($validVerToConclude=='VALID'){
            $this->updateVersion($row, 'CONCLUIDA');
        }

        return $validVerToConclude;
    }

    /**
     * Cancela conclusão versão selecionada na tela de
     * versão.
     * 
     * @param object $row
     * 
     * @return string|array 'VALID' caso a operação
     * seja concluida com sucesso, array de erros
     * caso não seja.
     */
    public function cancelConcludeVersion($row){
        $nrVersao = $row->get('NRVERSAOPROPCOM');

        $versao = $this->tco01100GeneralLibPropCom->getData('versaopropcom', ['NRVERSAOPROPCOM'=>$nrVersao]);
        if($versao && $versao[0]['IDVERSAOPROPCOM']=='CONCLUIDA'){
            $this->updateVersion($row, 'PENDENTE');
            return 'VALID';
        } else return $versao;

    }

    /**
     * Aprova versão selecionada
     * na tela de versão.
     * 
     * @param object $row
     * 
     * @return string|array 'VALID' caso a operação
     * seja concluida com sucesso, array de erros
     * caso não seja.
     */
    public function aproveVersion($row){
        $versaoData = $row->get('versaoData');
        $aproveData = $row->get('aproveData');
        $nrPropostaComercial = $versaoData['NRPROPOSTACOMERCIAL'];
        $nrVersaoPropCom = $versaoData['NRVERSAOPROPCOM'];
        $isValidAprove = $this->isValidOpenAprove($versaoData);
        $returnData = [];

        if($isValidAprove){
            $invalidServToAprove = $this->setFilialOnServAprove($aproveData, $nrPropostaComercial);
            
            if(!$this->tcoUtil->oldsizeof($invalidServToAprove)){
                $this->updateVersion(['NRVERSAOPROPCOM'=>$nrVersaoPropCom], 'APROVADA');
                $returnData = 'VALID';
            } else {
                $returnData['invalidServToAprove'] = $invalidServToAprove;
            }
        } else {
            $returnData['invalidVersaoToAprove'] = true;
        }

        return $returnData;
    }

    /**
     * Define filiais nos serviços escolhidos 
     * caso a operação seja valida, e retorna os
     * registros invalidos.
     * 
     * @param array $aproveServs Serviços a serem
     * usados.
     * @param int|float $nrPropostaComercial
     * 
     * @return array array de registros invalidos.
     */
    private function setFilialOnServAprove($aproveServs, $nrPropostaComercial){
        $invalidRegisters = [];

        $this->entityManager->beginTransaction();

        foreach ($aproveServs as $key => $serv) {
            if(!$this->isUsedFilialOnAnotherPropCom($serv['CDFILIAL'], $nrPropostaComercial)){
                $this->updateFilialOnServPropCom($serv);
            } else {
                $invalidRegisters[] = $serv;
            }
        }

        if(!$this->tcoUtil->oldsizeof($invalidRegisters)){
            $this->entityManager->commit();
        } else {
            $this->entityManager->rollback();
        }

        return $invalidRegisters;
    }

    /**
     * Grava registro de filial para serviço
     * no banco de dados.
     * 
     * @param array $serv 
     */
    private function updateFilialOnServPropCom($serv){
        $sql = "UPDATE SERVICOPROPCOM 
            SET CDFILIAL = :CDFILIAL 
            WHERE NRSERVICOPROPCOM = :NRSERVICOPROPCOM";

        $params = array(
            "CDFILIAL" => $serv['CDFILIAL'],
            "NRSERVICOPROPCOM" => $serv['NRSERVICOPROPCOM']
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

    /**
     * Verifica se a filial é utilizada em outra
     * proposta comercial ao salvar.
     * 
     * @param int|float $cdFilial
     * @param int|float $nrPropostaComercial
     * 
     * @return bool
     */
    private function isUsedFilialOnAnotherPropCom($cdFilial, $nrPropostaComercial){
        $sql = "SELECT
                    SPC.CDFILIAL
                FROM
                    SERVICOPROPCOM SPC
                    INNER JOIN VERSAOPROPCOM VPC
                        ON VPC.NRVERSAOPROPCOM = SPC.NRVERSAOPROPCOM
                WHERE
                    VPC.NRPROPOSTACOMERCIAL <> :NRPROPOSTACOMERCIAL
                    AND SPC.CDFILIAL = :CDFILIAL";

        $params = array(
            "CDFILIAL" => $cdFilial,
            "NRPROPOSTACOMERCIAL" => $nrPropostaComercial
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
                
        return (bool)$this->tcoUtil->oldsizeof($result);
    }

    /**
     * Cancela aprovação versão selecionada na tela de
     * versão.
     * 
     * @param object $row
     * 
     * @return string|array 'VALID' caso a operação
     * seja concluida com sucesso, array de erros
     * caso não seja.
     */
    public function cancelAproveVersion($row){
        $result = null;
        $nrVersao = $row['NRVERSAOPROPCOM'];
        $versao = $this->tco01100GeneralLibPropCom->getData('versaopropcom_completa', array(
            'NRVERSAOPROPCOM'=>$nrVersao
        ));

        if($this->tcoUtil->oldsizeof($versao) && $versao[0]['SITVERSAO']=='APROVADA'){
            $this->updateVersion($row, 'CONCLUIDA');
            $this->clearFilialFromServ($nrVersao);
            $result = 'VALID';
        } else {
            $result = ['invalidSit' => true];
        }

        return $result;
    }
    
    /**
     * Limpa registro de filial de serviços 
     * pertencentes a proposta comercial.
     * 
     * @param int|float $nrVersao
     */
    private function clearFilialFromServ($nrVersao){
        try{
            $sql = "UPDATE SERVICOPROPCOM SET CDFILIAL = NULL WHERE NRVERSAOPROPCOM = :NRVERSAOPROPCOM";
            
            $params = array(
                "NRVERSAOPROPCOM" => $nrVersao
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * Altera situação versão no banco de dados
     * 
     * @param ArrayObject $row
     * @param string $state
     * 
     */
    private function updateVersion($row, $state){
        try{
            $sql = "UPDATE VERSAOPROPCOM SET IDVERSAOPROPCOM=:IDVERSAOPROPCOM where NRVERSAOPROPCOM = :NRVERSAOPROPCOM";
            
            $params = array(
                "NRVERSAOPROPCOM" => $row['NRVERSAOPROPCOM'],
                "IDVERSAOPROPCOM" => $state
            );
            
            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se versão é valida para concluir
     * 
     * @param object $row
     * 
     * @return string|array 'VALID' caso seja
     * valida, serviços invalidos caso não seja
     */
    private function isValidVersaoToConclude($row){
        $nrVersao = $row->get('NRVERSAOPROPCOM');
        $validVersToConclude = [];
        $servVersaoPropCom = $this->getServToConcludeVersion($nrVersao);

        $invalidServ = [];
        foreach ($servVersaoPropCom as $key => $servico) {
            if($servico['NRCARDAPIOTRAB']){
                if($servico['IDSERVICOPROPCOM']=='AGUARDANDOPLANEJAMENTO'){
                    $invalidServ[] = $servico;
                }
            } else {
                $invalidServ[] = $servico;
            }
        }
        if($this->tcoUtil->oldsizeof($invalidServ)) $validVersToConclude['invalidServ'] = $invalidServ;

        $versao = $this->tco01100GeneralLibPropCom->getData('versaopropcom_completa', ['NRVERSAOPROPCOM'=>$nrVersao]);

        if(!$versao || $versao[0]['SITVERSAO']!='CALCULADA'){
            $validVersToConclude['invalidVer'] = true;
        }

        return $this->tcoUtil->oldsizeof($validVersToConclude) ? $validVersToConclude : 'VALID';
    }

    /**
     * Retorna serviços para verificação de validade de
     * conclusao da versao da proposta.
     * 
     * @param int|float $nrVersaoPropCom
     * 
     * @return array registros de serviço.
     */
    private function getServToConcludeVersion($nrVersaoPropCom){
        try{
            $sql = "SELECT DISTINCT
                        SP.NMSERVICOPROPOSTA,
                        SPC.NRSERVICOPROPCOM,
                        SPC.IDSERVICOPROPCOM,
                        SPC.NRVERSAOPROPCOM,
                        DSPC.NRCARDAPIOTRAB
                    FROM
                        SERVICOPROPCOM SPC
                        LEFT JOIN SERVICOPROPOSTA SP ON SP.NRSERVICOPROPOSTA = SPC.NRSERVICOPROPOSTA
                        LEFT JOIN DETALHESERVPROPCOM DSPC ON DSPC.NRDETALHESERVPROPCOM = SPC.NRDETALHESERVPROPCOM
                    WHERE
                        SPC.NRVERSAOPROPCOM = :NRVERSAOPROPCOM";
            
            $params = array(
                "NRVERSAOPROPCOM" => $nrVersaoPropCom
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            $result = $query->fetchAll();

            return $result;
        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se proposta comercial não possui nenhuma versão 
     * já aprovada.
     * 
     * @param DTO\Row $row
     * 
     * @return bool validade de operação
     * de aprovação;
     */
    public function isValidOpenAprove($row){
        $isAproved = false;
        $allVersions =  $this->tco01100GeneralLibPropCom->getData('versaopropcom', array(
            'NRPROPOSTACOMERCIAL' => $row['NRPROPOSTACOMERCIAL']
        ));

        $curVersion =  $this->tco01100GeneralLibPropCom->getData('versaopropcom_completa', array(
            'NRVERSAOPROPCOM' => $row['NRVERSAOPROPCOM']
        ));
        $validSit = $this->tcoUtil->oldsizeof($curVersion) ? $curVersion[0]['SITVERSAO']=='CONCLUIDA' : false;

        $isValidSitPropCom = $this->isValidPropComSitNewVersion($row['NRPROPOSTACOMERCIAL']);
        
        foreach ($allVersions as $key => $ver) {
            if($ver['IDVERSAOPROPCOM']=='APROVADA'){
                $isAproved = true;
            }
        }

        return !$isAproved && $validSit && $isValidSitPropCom;
    }

    /**
     * Duplica versão para rows selecionadas.
     * 
     * @param array $rows Array de rows a serem
     * duplicadas.
     * 
     * @return string validade da operação.
     */
    public function duplicateVersion($rows){
        $this->entityManager->beginTransaction();
        try{
            foreach ($rows as $key => $row) {
                $procedure = new StoredProcedure($this->connection, 'DUPLICA_VERSAOPROPCOM');
                $procedure->addParam(new Param('P_NRVERSAOPROPCOM',  Param::PARAM_INPUT,  $row->get('NRVERSAOPROPCOM'),        Param::PARAM_TYPE_INT));
                $procedure->addParam(new Param('P_NRORG',            Param::PARAM_INPUT,  $this->environment->getNrOrg(),      Param::PARAM_TYPE_INT));
                $procedure->addParam(new Param('P_CDOPERADOR',       Param::PARAM_INPUT,  $this->environment->getCdoperador(), Param::PARAM_TYPE_STR));
                $procedure->execute();
            }
            $this->entityManager->commit();
            return 'VALID';

        } catch (\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }

    /**
     * Retorna proposta comercial de acordo com
     * numero da versão.
     * 
     * @param int|float $nrVersao
     * 
     * @return array proposta comercial
     */
    public function getPropComByVersao($nrVersao){
        $sql = "SELECT
                    PC.*, SIT_PROPCOMERCIAL(PC.NRPROPOSTACOMERCIAL) AS SITUACAO
                FROM
                    PROPOSTACOMERCIAL PC
                WHERE
                    PC.NRPROPOSTACOMERCIAL IN (
                        SELECT NRPROPOSTACOMERCIAL
                        FROM VERSAOPROPCOM
                        WHERE NRVERSAOPROPCOM =:NRVERSAOPROPCOM)";

        $params = [
            "NRVERSAOPROPCOM"=>$nrVersao
        ];

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();       
    }

    /**
     * Verifica situação da proposta comercial
     * para adição de versão.
     * 
     * @param int|float $nrPropostaComercial
     */
    public function isValidPropComSitNewVersion($nrPropostaComercial){
        $sql = "SELECT
                PC.*, SIT_PROPCOMERCIAL(PC.NRPROPOSTACOMERCIAL) AS SITUACAO
            FROM
                PROPOSTACOMERCIAL PC
            WHERE
                PC.NRPROPOSTACOMERCIAL = :NRPROPOSTACOMERCIAL";

        $params = [
            "NRPROPOSTACOMERCIAL"=>$nrPropostaComercial
        ];

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        
        return $this->tcoUtil->oldsizeof($result) ? $result[0]['SITUACAO']=='PENDENTE' : false;
    }
    
    public function procedureImportServicos($row) {
        
        $procedure = new StoredProcedure($this->connection, 'IMPORTA_SERVICOPROPCOM');
		$procedure->addParam(new Param('P_NRPROPOSTACOMERCIALORI', Param::PARAM_INPUT,  $row['NRPROPOSTACOMERCIAL'],         Param::PARAM_TYPE_INT));
		$procedure->addParam(new Param('P_NRVERSAOPROPCOMORI',     Param::PARAM_INPUT,  $row['NRVERSAOPROPCOM'],             Param::PARAM_TYPE_INT));
		$procedure->addParam(new Param('P_SERVICOSORIGEM',         Param::PARAM_INPUT,  $row['L_SERVICOPROPOSTA'],           Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NRVERSAOPROPCOMDEST',    Param::PARAM_INPUT,  $row['NRVERSAOPROPCOMDEST'],         Param::PARAM_TYPE_INT));
		$procedure->addParam(new Param('P_COMPLEMENTAR',           Param::PARAM_INPUT,  $row['IMPORTAR_COMPLEMENTAR'],       Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDOPERADOR',             Param::PARAM_INPUT,  $this->environment->getCdoperador(), Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NRORG',                  Param::PARAM_INPUT,  $this->environment->getNrOrgTrab(),  Param::PARAM_TYPE_INT));
		$procedure->addParam(new Param('P_CDFILIALLOGADA',         Param::PARAM_INPUT,  $this->environment->getCdFilial(),   Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NRLISTAERROS',           Param::PARAM_OUTPUT, null,                                Param::PARAM_TYPE_INT, 20));
        
        $result = $procedure->execute();
        
        return $result;
        
    }
    
    public function enviaEmailNotificacao(DTO\Request $request, DTO\Response $response){
        $this->tco01100GeneralLibPropCom->enviaEmail($request, $response);
    }
    
    public function getAllTypeDish($nrservicopropcom){
        $sql = "SELECT CG.CDGRTIPOPR, MAX(CG.NRPORCIONAMENTO) NRPORCIONAMENTO, MAX(CG.NRGRAMATURACARDBASE) AS NRGRAMATURACARDBASE,
               MAX(CG.IDTIPOGRAMATURA) AS IDTIPOGRAMATURA, S.NRCARDAPIOBASE
          FROM SERVICOPROPCOM S
               INNER JOIN DETALHESERVPROPCOM D ON D.NRDETALHESERVPROPCOM = S.NRDETALHESERVPROPCOM
               INNER JOIN CONFIGSERVPROPCOM C ON C.NRDETALHESERVPROPCOM = D.NRDETALHESERVPROPCOM
               INNER JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = C.CDTIPOPRAT
               INNER JOIN CONFGRUPTPCARDBASE CG ON CG.NRCARDAPIOBASE = S.NRCARDAPIOBASE AND CG.CDGRTIPOPR = TP.CDGRTIPOPR
               LEFT JOIN CONFGRUPOTPPROPCOM CGP ON CGP.NRDETALHESERVPROPCOM = D.NRDETALHESERVPROPCOM AND
                                                   CGP.CDGRTIPOPR = CG.CDGRTIPOPR 
         WHERE S.NRSERVICOPROPCOM = :NRSERVICOPROPCOM
           AND CGP.NRCONFGRUPOTPPROPCOM IS NULL
         GROUP BY CG.CDGRTIPOPR, S.NRCARDAPIOBASE";


        $params = array(
            "NRSERVICOPROPCOM" => $nrservicopropcom
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $retorno = $query->fetchAll();

        return $retorno;
    }
    
    public function allPorcionamentos($nrservicopropcom){
        $sql = "SELECT DISTINCT
                grtp.cdgrtipopr,
                grtp.nmgrtipopr,
                grtp.nrorg,
                grtp.idativo,
                cgtpspc.nrconfgrupotppropcom,
                cgtpspc.nrporcionamento,
                pc.cdporcionamento,
                pc.nmporcionamento,
                cgtpspc.nrgramaturacardbase,
                gmcb.vrgramatura,
                gmcb.sgunidade,
                cgtpspc.idtipogramatura,
                spc.nrdetalheservpropcom,
                spc.nrservicopropcom
            FROM
                configservpropcom    cspc
                LEFT JOIN detalheservpropcom   dtspc ON cspc.nrdetalheservpropcom = dtspc.nrdetalheservpropcom
                LEFT JOIN servicopropcom       spc ON spc.nrservicopropcom = dtspc.nrservicopropcom
                                                AND spc.nrdetalheservpropcom = dtspc.nrdetalheservpropcom
                LEFT JOIN tipoprat             tp ON tp.cdtipoprat = cspc.cdtipoprat
                LEFT JOIN grtipopr             grtp ON grtp.cdgrtipopr = tp.cdgrtipopr
                LEFT JOIN confgrupotppropcom   cgtpspc ON ( cgtpspc.nrdetalheservpropcom = dtspc.nrdetalheservpropcom
                                                          AND cgtpspc.cdgrtipopr = grtp.cdgrtipopr )
                LEFT JOIN porcionamento        pc ON pc.nrporcionamento = cgtpspc.nrporcionamento
                LEFT JOIN gramaturacardbase    gmcb ON gmcb.nrgramaturacardbase = cgtpspc.nrgramaturacardbase
                LEFT JOIN configcardbase ON configcardbase.nrconfigcardbase = cspc.nrconfigcardbase
            WHERE
                spc.nrservicopropcom = :nrservicopropcom
                AND configcardbase.idclassificacao = 'ALIMENTAR'";


        $params = array(
            "NRSERVICOPROPCOM" => $nrservicopropcom
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $retorno = $query->fetchAll();

        return $retorno;
    }

    public function getStandardTypeDish( $NRCARDAPIOBASE ){
        $sql = <<<SQL
            SELECT
                NRCARDAPIOBASE,
                NRCONFIGCARDBASE,
                CDTIPOPRAT,
                NVL(DSCOMPLEMENTO, ' ') AS DSCOMPLEMENTO,
                IDCLASSIFICACAO,
                IDTEMPERATURA,
                NRORDEM,
                IDPADRAO,
                IDATIVO
            FROM
                CONFIGCARDBASE
            WHERE
                NRCARDAPIOBASE = :NRCARDAPIOBASE AND
                IDPADRAO = 'S'
            ORDER BY
                NRORDEM
SQL;
        $stmt = $this->connection->prepare($sql);
        $stmt->execute([
            'NRCARDAPIOBASE' => $NRCARDAPIOBASE
        ]);
        return $stmt->fetchAll();
    }

    public function insertConfigServPropCom( $params ){
        $operador = $this->environment->getCdOperador();
        $nrorg = $this->environment->getNrOrg();
        $nrConfigServPropCom  = $this->newCode->getCodeByTableName('CONFIGSERVPROPCOM', $nrorg, 1, 10, true);
        $sql = <<<SQL
        INSERT INTO CONFIGSERVPROPCOM
           (
                NRCONFIGSERVPROPCOM,  NRDETALHESERVPROPCOM,  NRCONFIGCARDBASE,  CDTIPOPRAT,
                DSCOMPLEMENTO,        NRORDEM,               IDTEMPERATURA,     NRORG, 
                IDATIVO,              DTINCLUSAO,            DTULTATU,          NRORGINCLUSAO,
                CDOPERINCLUSAO,       NRORGULTATU,           CDOPERULTATU
            )
            VALUES (
                $nrConfigServPropCom, :NRDETALHESERVPROPCOM, :NRCONFIGCARDBASE, :CDTIPOPRAT,
                :DSCOMPLEMENTO,       :NRORDEM,              :IDTEMPERATURA,    $nrorg, 
                :IDATIVO,             SYSDATE,               NULL,              $nrorg,
                $operador,            NULL,                  NULL
            )
SQL;
        $stmt = $this->connection->prepare($sql);
        $stmt->execute([
            'NRDETALHESERVPROPCOM' => $params['NRDETALHESERVPROPCOM'],
            'NRCONFIGCARDBASE'     => $params['NRCONFIGCARDBASE'],
            'CDTIPOPRAT'           => $params['CDTIPOPRAT'],
            'DSCOMPLEMENTO'        => $params['DSCOMPLEMENTO'],
            'NRORDEM'              => $params['NRORDEM'],
            'IDTEMPERATURA'        => $params['IDTEMPERATURA'],
            'IDATIVO'              => $params['IDATIVO']
        ]);
    }

    public function insertConfGrupoTPPropCom( $params ){
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $NRCONFGRUPOTPPROPCOM = $this->newCode->getCodeByTableName('CONFGRUPOTPPROPCOM', $nrorg, 1, 10, true);
        $sql = <<<SQL
        INSERT INTO CONFGRUPOTPPROPCOM
        (
            NRCONFGRUPOTPPROPCOM,  NRDETALHESERVPROPCOM,  CDGRTIPOPR,     NRPORCIONAMENTO,
            NRGRAMATURACARDBASE,    NRORG,                 IDATIVO,        DTINCLUSAO,
            DTULTATU,              NRORGINCLUSAO,         CDOPERINCLUSAO, NRORGULTATU,
            CDOPERULTATU,          IDTIPOGRAMATURA
        )
        VALUES
        (
            :NRCONFGRUPOTPPROPCOM, :NRDETALHESERVPROPCOM, :CDGRTIPOPR,    :NRPORCIONAMENTO,
            :NRGRAMATURACARDBASE,   $nrorg,                :IDATIVO,       SYSDATE,
            NULL,                  $nrorg,                $cdoperador,    NULL,
            NULL,                  :IDTIPOGRAMATURA
        )
SQL;
    
        $stmt = $this->connection->prepare($sql);
        $stmt->execute([
            'NRCONFGRUPOTPPROPCOM' => $NRCONFGRUPOTPPROPCOM,
            'NRDETALHESERVPROPCOM' => $params['NRDETALHESERVPROPCOM'],
            'CDGRTIPOPR'           => $params['CDGRTIPOPR'],
            'NRPORCIONAMENTO'      => $params['NRPORCIONAMENTO'],
            'NRGRAMATURACARDBASE'  => $params['NRGRAMATURACARDBASE'],
            'IDATIVO'              => $params['IDATIVO'],
            'IDTIPOGRAMATURA'      => $params['IDTIPOGRAMATURA']
        ]);
    }

    private function getAllRulesGroupMenu( $NRCARDAPIOBASE ){
        $sql = <<<SQL
            SELECT 
                CDGRTIPOPR, 
                NRPORCIONAMENTO, 
                NRGRAMATURACARDBASE, 
                IDTIPOGRAMATURA, 
                IDATIVO
            FROM
                CONFGRUPTPCARDBASE
            WHERE
                NRCARDAPIOBASE = $NRCARDAPIOBASE
SQL;
        $stmt = $this->connection->prepare($sql);
        $stmt->execute();
        $dataSet = $stmt->fetchAll();
        $mapRules = [];
        foreach($dataSet as $item){
            $mapRules[$item['CDGRTIPOPR']] = $item;
        }
        return $mapRules;
    }

    private function getAllGroupConfGrupoTPPropCom( $NRDETALHESERVPROPCOM ){
        $sql = <<<SQL
            SELECT
                CDGRTIPOPR
            FROM 
                CONFGRUPOTPPROPCOM
            WHERE
                NRDETALHESERVPROPCOM = $NRDETALHESERVPROPCOM
SQL;
        $stmt = $this->connection->prepare($sql);
        $stmt->execute();
        $typeGroup = $stmt->fetchAll();
        $allGroup = array_column($typeGroup, 'CDGRTIPOPR');
        $mapGroup = [];
        foreach($allGroup as $group){
            $mapGroup[$group] = '';
        }
        return $mapGroup;
    }

    public function inserePorcionamentoGramaturaProposta( $params ){
        //$allTypeDish = $this->getAllTypeDish($params['NRSERVICOPROPCOM']);
        $allGroupTypeDish = $this->getAllTypeDish($params['NRSERVICOPROPCOM']);
        $groups = $this->getAllGroupConfGrupoTPPropCom($params['NRDETALHESERVPROPCOM']);
        $NRCARDAPIOBASE = !empty($allGroupTypeDish) ? $allGroupTypeDish[0]['NRCARDAPIOBASE'] : '';
        
        if(isset($NRCARDAPIOBASE) && !empty($NRCARDAPIOBASE)){
            $mapRules = $this->getAllRulesGroupMenu($NRCARDAPIOBASE);    
        }else{
            $mapRules = [];    
        }
        
        foreach( $allGroupTypeDish as $typeDish ) {
            $CDGRTIPOPR = $typeDish['CDGRTIPOPR'];
            if(!isset($groups[$CDGRTIPOPR]) && isset($mapRules[$CDGRTIPOPR])){
                $mapRules[$CDGRTIPOPR]['NRDETALHESERVPROPCOM'] = $params['NRDETALHESERVPROPCOM'];
                $this->insertConfGrupoTPPropCom($mapRules[$CDGRTIPOPR]);
            }
        }
    }
}